"""
This qidicom module contains utilities to read and edit DICOM files.
"""

__version__ = '2.1.1'
"""
The one-based major.minor.patch version.
The version numbering scheme loosely follows http://semver.org/.
The major version is incremented when there is an incompatible
public API change. The minor version is incremented when there
is a backward-compatible functionality change. The patch version
is incremented when there is a backward-compatible refactoring
or bug fix. All major, minor and patch version numbers begin at
1.
"""



import os
from qiutil.logging import logger
from . import reader

def edit(*in_files, **opts):
    """
    Edits the given DICOM files.

    :param in_files: the input DICOM files or directories containing
        DICOM files
    :param opts: the following options:
    :keyword dest: the directory in which to write the modified DICOM
        files (default is to edit in-place)
    :return: the output file paths
    """
    # Prepare the destination directory.
    dest = opts.get('dest')
    if dest:
        dest = os.path.abspath(dest)
        if not os.path.exists(dest):
            os.makedirs(dest)
    else:
        dest = os.getcwd()
    
    # The output file paths.
    out_files = []
    # Open the DICOM store on each DICOM file (skipping non-DICOM files),
    # yield to the edit callback and save to the file in the destination
    # directory.
    for ds in reader.iter_dicom(*dicom_files):
        logger(__name__).debug("Editing the DICOM file %s..." % ds.filename)
        yield ds
        _, fname = os.path.split(ds.filename)
        out_file = os.path.join(dest, fname)
        ds.save_as(out_file)
        out_files.append(out_file)
        logger(__name__).debug("Saved the edited DICOM file as %s." % out_file)
    logger(__name__).info("The edited DICOM files were saved in %s." % dest)



# Absolute import (standard in Python 3) imports dicom from pydicom
# rather than the parent module.
from __future__ import absolute_import
import os
import re
import operator
from dicom import datadict
from qiutil.logging import logger
from . import (reader, writer)

# Uncomment to debug pydicom.
# import dicom
# dicom.debug(True)

def select(ds, *tags):
    """
    Reads the given DICOM dataset tags.
    
    :param ds: the pydicom dicom object
    :param tags: the names of tags to read (default all unbracketed tags)
    :return: the tag name => value dictionary
    """
    if not tags:
        # Skip tags with a bracketed name.
        tags = (de.name for de in ds if de.name[0] != '[')
    tdict = {}
    for t in tags:
        try:
            # The tag attribute.
            tattr = re.sub('\W', '', t)
            # Collect the tag value.
            tdict[t] = operator.attrgetter(tattr)(ds)
        except AttributeError:
            pass
    
    return tdict


def edit(*in_files, **opts):
    """
    Sets the tags of the given DICOM files.

    :param in_files: the input DICOM files or directories containing
        DICOM files
    :param opts: the DICOM header (I{name}, I{value}) tag values to set
        and the following option:
    :return: the modified file paths
    """
    dest = opts.pop('dest', None)
    # The {tag: value} dictionary.
    tv = {datadict.tag_for_name(t.replace(' ', ''))
                          : v for t, v in opts.iteritems()}
    # The {tag: VR} dictionary.
    tvr = {t: datadict.get_entry(t)[0] for t in tv.iterkeys()}

    # Edit the files
    logger(__name__).info("Editing the DICOM files with the following tag"
                          " values: %s..." % tag_values)
    for ds in writer.edit(*in_files):
        # Set the tag values.
        for t, v in tv.iteritems():
            if t in ds:
                ds[t].value = v
            else:
                ds.add_new(t, tvr[t], v)

    # The output file path formatter.
    format_out_file = lambda f: os.path.join(dest, os.path.split(f)[1])

    # Return the output file paths.
    return [format_out_file(f) for f in in_files]



import os
import glob
from nose.tools import (assert_equal, assert_true)
from qiutil.logging import logger
from qidicom import (reader, meta)
from .. import ROOT

FIXTURE = os.path.join(ROOT, 'fixtures', 'dicom')
"""The test image parent directory."""

SBJ_ID = 'Sarcoma002'
"""The Subject ID."""

STUDY_ID = '1'
"""The test image Study ID."""

STUDY_UID = '1.3.12.2.1107.5.2.32.35139.30000010011316342567100000106'
"""The test image Study UID."""

SERIES_NBR = 11
"""The test image
 Series Number."""

SERIES_UID = '1.3.12.2.1107.5.2.32.35139.2010011914134225154552501.0.0.0'
"""The test image Series UID."""

INSTANCE_NBR = 6
"""The test image Instance Number."""


class TestReader(object):
    """The dicom reader unit tests."""

    def test_read_headers(self):
        # The first brain image.
        files = glob.glob(FIXTURE + '/*')
        # Read the tags.
        for ds in reader.iter_dicom_headers(FIXTURE):
            tdict = meta.select(ds, 'Study ID', 'Series Number')
            study = tdict['Study ID']
            assert_equal(study, STUDY_ID, "Study tag incorrect: %s" % study)
            series = tdict['Series Number']
            assert_equal(series, SERIES_NBR, "Series tag incorrect: %d" % series)


if __name__ == "__main__":
    import nose
    nose.main(defaultTest=__name__)



import os
import glob
import shutil
from nose.tools import (assert_equal, assert_true)
from dicom import datadict as dd
from qidicom import (reader, meta)
from qiutil.logging import logger
from .. import ROOT

FIXTURE = os.path.join(ROOT, 'fixtures', 'dicom')
"""The test fixture."""

RESULTS = os.path.join(ROOT, 'results', 'fixtures', 'dicom')
"""The test results directory."""


class TestMeta(object):
    """
    The dicom meta unit tests.
    
    :Note: these tests also indirectly test the writer module.
    """

    def setUp(self):
        shutil.rmtree(RESULTS, True)

    def tearDown(self):
        shutil.rmtree(RESULTS, True)

    def test_edit_metadata(self):
        # The tag name => value map.
        tnv = dict(PatientID='Test Patient', BodyPartExamined='HIP')

        # The tag => value map.
        tv = {dd.tag_for_name(name): value for name, value in tnv.iteritems()}

        # Edit the headers.
        files = set(meta.edit(RESULTS, FIXTURE, **tnv))

        # Verify the result.
        for ds in reader.iter_dicom(RESULTS):
            assert_true(ds.filename in files)
            for t, v in tv.iteritems():
                assert_equal(v, ds[t].value)


if __name__ == "__main__":
    import nose
    nose.main(defaultTest=__name__)



from collections import defaultdict
from ..dictionary_hierarchy import DictionaryHierarchy
from . import (reader, meta)


def read_hierarchy(*files):
    """
    Returns the ImageHierarchy for the DICOM files in the given locations.

    :param files: the files or directories to walk for DICOM files
    :return: the image hierarchy
    :rtype: :class:`qiutil.image_hierarchy.ImageHierarchy`
    """
    return ImageHierarchy(*files)


class ImageHierarchy(DictionaryHierarchy):
    """
    ImageHierarchy wraps the DICOM image subject-study-series-image hierarchy.
    """

    TAGS = ('Patient ID', 'Study Instance UID', 'Series Instance UID',
            'Instance Number')

    def __init__(self, *files):
        """
        :param the input DICOM files
        """
        
        # the subject: series: image nested dictionary
        self.tree = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))
        super(ImageHierarchy, self).__init__(self.tree)
        for ds in reader.iter_dicom_headers(*files):
            self.add(ds)

    def add(self, ds):
        """
        Adds the subject-study-series-image hierarchy entries from the given
        DICOM dataset.

        :param ds: the DICOM dataset
        """
        # build the image hierarchy
        tdict = meta.select(ds, *ImageHierarchy.TAGS)
        path = [tdict[t] for t in ImageHierarchy.TAGS]
        self.tree[path[0]][path[1]][path[2]].append(path[3])



import os
import glob
from nose.tools import (assert_equal, assert_true)
import qidicom
from qiutil.logging import logger
from .. import ROOT

FIXTURE = os.path.join(ROOT, 'fixtures', 'dicom')
"""The test image parent directory."""

SBJ_ID = 'Sarcoma002'
"""The Subject ID."""

STUDY_ID = '1'
"""The test image Study ID."""

STUDY_UID = '1.3.12.2.1107.5.2.32.35139.30000010011316342567100000106'
"""The test image Study UID."""

SERIES_NBR = 11
"""The test image Series Number."""

SERIES_UID = '1.3.12.2.1107.5.2.32.35139.2010011914134225154552501.0.0.0'
"""The test image Series UID."""

INSTANCE_NBR = 6
"""The test image Instance Number."""


class TestHierarchy(object):
    """The dicom hierarchy unit tests."""

    def test_hierarchy(self):
        paths = list(qidicom.hierarchy.read_hierarchy(FIXTURE))
        assert_equal(len(paths), 1,
                     "The DICOM Helper image hierarchy path count is incorrect")
        path = paths[0]
        assert_equal(len(path), 4,
                     "The DICOM Helper image path item count is incorrect")
        sbj_id, study_uid, series_uid, inst_nbr = path
        assert_equal(sbj_id, SBJ_ID, "Subject ID incorrect: %s" % sbj_id)
        assert_equal(study_uid, STUDY_UID,
                     "Study UID incorrect: %s" % study_uid)
        assert_equal(series_uid, SERIES_UID,
                     "Series UID incorrect: %s" % series_uid)
        assert_equal(inst_nbr, INSTANCE_NBR,
                     "Instance Number incorrect: %s" % inst_nbr)


if __name__ == "__main__":
    import nose
    nose.main(defaultTest=__name__)



# Absolute import (standard in Python 3) imports dicom from pydicom
# rather than the parent module.
from __future__ import absolute_import
import os
import dicom
from dicom.filereader import InvalidDicomError
import qiutil
from qiutil.logging import logger


def iter_dicom(*dicom_files):
    """
    Iterates over the DICOM data sets for DICOM files at the given locations.
    
    :param dicom_files: the DICOM files or directories containing DICOM files
    """
    return DicomIterator(*dicom_files)


def iter_dicom_headers(*dicom_files):
    """
    Iterates over the DICOM headers for DICOM files at the given locations.
    
    :param dicom_files: the DICOM files or directories containing DICOM files
    """
    return DicomHeaderIterator(*dicom_files)


class DicomIterator(qiutil.file.FileIterator):
    """
    DicomIterator is a generator class for reading the pydicom data sets from
    DICOM files.
    """

    def __init__(self, *dicom_files, **opts):
        """
        :param dicom_files: the DICOM files to include
        """
        super(DicomIterator, self).__init__(*dicom_files)
        self.opts = opts

    def next(self):
        """
        Iterates over each DICOM data set.
        
        :yield: the next pydicom dicom object
        """
        for filename in super(DicomIterator, self).next():
            with qiutil.file.open(filename) as fp:
                try:
                    yield dicom.read_file(fp, **self.opts)
                except InvalidDicomError:
                    logger(__name__).info("Skipping non-DICOM file %s" % filename)


class DicomHeaderIterator(DicomIterator):
    """
    DicomHeaderIterator is a generator class for reading the pydicom non-pixel
    data sets from DICOM files.
    """

    OPTS = dict(defer_size=256, stop_before_pixels=True, force=False)

    def __init__(self, *dicom_files):
        super(DicomHeaderIterator, self).__init__(*dicom_files, **DicomHeaderIterator.OPTS)
